﻿using System;
using System.Threading;

using wclCommon;
using wclSerial;

namespace UsbMonitor
{
    class Program
    {
        private class UsbMonitor
        {
            private ManualResetEvent TerminateEvent;
            private Thread Thread;

            private void Monitor_OnRemoved(Object Sender, wclUsbDevice Device)
            {
                Console.WriteLine("Device REMOVED: ");
                Console.WriteLine("  Friendly name: " + Device.FriendlyName);
                Console.WriteLine("  Hardware ID: " + Device.HardwareId);
                Console.WriteLine("  Instance: " + Device.Instance);
                Console.WriteLine("  Manufacturer: " + Device.Manufacturer);
            }

            private void Monitor_OnInserted(Object Sender, wclUsbDevice Device)
            {
                Console.WriteLine("Device INSERTED: ");
                Console.WriteLine("  Friendly name: " + Device.FriendlyName);
                Console.WriteLine("  Hardware ID: " + Device.HardwareId);
                Console.WriteLine("  Instance: " + Device.Instance);
                Console.WriteLine("  Manufacturer: " + Device.Manufacturer);
            }

            private void ThreadProc()
            {
                wclUsbMonitor Monitor = new wclUsbMonitor();
                Monitor.OnInserted += Monitor_OnInserted;
                Monitor.OnRemoved += Monitor_OnRemoved;

                Console.WriteLine("Enumerate installed USB devices");
                wclUsbDevice[] Devices;
                Int32 Res = Monitor.EnumDevices(out Devices);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    Console.WriteLine("Failed to enumerate USB devices: 0x" + Res.ToString("X8"));
                else
                {
                    if (Devices == null || Devices.Length == 0)
                        Console.WriteLine("No USB devices were found");
                    else
                    {
                        int i = 0;
                        foreach (wclUsbDevice Device in Devices)
                        {
                            Console.WriteLine("Device " + i.ToString());
                            Console.WriteLine("  Friendly name: " + Device.FriendlyName);
                            Console.WriteLine("  Hardware ID: " + Device.HardwareId);
                            Console.WriteLine("  Instance: " + Device.Instance);
                            Console.WriteLine("  Manufacturer: " + Device.Manufacturer);
                            i++;
                        }
                    }
                }

                Console.WriteLine("Try tp start monitoring USB device.");
                Res = Monitor.Start();
                if (Res != wclErrors.WCL_E_SUCCESS)
                    Console.WriteLine("Failed to start monitoring USB devices 0x" + Res.ToString("X8"));
                else
                    Console.WriteLine("Monitoring started");

                wclMessageBroadcaster.Wait(TerminateEvent);

                if (Monitor.Monitoring)
                {
                    Console.WriteLine("Try to stop monitoring");
                    Res = Monitor.Stop();
                    if (Res != wclErrors.WCL_E_SUCCESS)
                        Console.WriteLine("Failed to stop monitoring USB devices 0x" + Res.ToString("X8"));
                    else
                        Console.WriteLine("Monitoring stopped");
                }
                Monitor = null;
            }

            public UsbMonitor()
            {
                TerminateEvent = null;
                Thread = null;
            }

            public void Start()
            {
                TerminateEvent = new ManualResetEvent(false);
                Thread = new Thread(ThreadProc);
                Thread.Start();
            }

            public void Stop()
            {
                TerminateEvent.Set();
                Thread.Join();
                Thread = null;

                TerminateEvent.Close();
                TerminateEvent = null;
            }
        };

        static void Main(string[] args)
        {
            wclMessageBroadcaster.SetSyncMethod(wclMessageSynchronizationKind.skApc);

            Console.Clear();
            Console.WriteLine("USB MONITOR DEMO");
            Console.WriteLine("===================");
            Console.WriteLine("The demo shows how to enumerate and monitor USB devices.");

            UsbMonitor Monitor = new UsbMonitor();
            Monitor.Start();
            Console.WriteLine("Press ENTER to stop");
            Console.ReadLine();
            Monitor.Stop();
            Monitor = null;
        }
    }
}
