﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using wclCommon;
using wclCommunication;
using wclSerial;

namespace SerialClient
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private wclSerialClient wclSerialClient;
        private wclSerialMonitor wclSerialMonitor;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void fmMain_Loaded(object sender, RoutedEventArgs e)
        {
            wclSerialMonitor = new wclSerialMonitor();

            wclSerialClient = new wclSerialClient();
            wclSerialClient.OnConnect += new wclClientConnectionConnectEvent(wclSerialClient_OnConnect);
            wclSerialClient.OnData += new wclConnectionDataEvent(wclSerialClient_OnData);
            wclSerialClient.OnDisconnect += new wclClientConnectionDisconnectEvent(wclSerialClient_OnDisconnect);
            wclSerialClient.OnError += new wclSerialDeviceErrorEvent(wclSerialClient_OnError);
            wclSerialClient.OnReadError += new wclSerialDeviceReadErrorEvent(wclSerialClient_OnReadError);
            wclSerialClient.OnEvents += new wclSerialDeviceEventsEvent(wclSerialClient_OnEvents);

            EnumComPorts();

            ClearConfig();
            ClearTiemouts();
            ClearBuffers();

            edWriteTimeout.Text = wclSerialClient.WriteTimeout.ToString();

            cbFunction.SelectedIndex = 0;
            cbLineFeed.SelectedIndex = 0;
        }

        void wclSerialClient_OnConnect(object Sender, int Error)
        {
            if (Error == wclErrors.WCL_E_SUCCESS)
            {
                lbEvents.Items.Add("Connected to Serial Device: " + wclSerialClient.DeviceName);

                ReadConfiguration();
                ReadTiemouts();
                ReadBuffers();
            }
            else
                lbEvents.Items.Add("Connect error: 0x" + Error.ToString("X8"));
        }

        void wclSerialClient_OnData(object Sender, byte[] Data)
        {
            if (Data != null && Data.Length > 0)
            {
                String Str = Encoding.ASCII.GetString(Data);

                lbEvents.Items.Add("Received: " + Str);
            }
        }

        void wclSerialClient_OnDisconnect(object Sender, int Reason)
        {
            lbEvents.Items.Add("Disconnected: 0x" + Reason.ToString("X8"));

            ClearConfig();
            ClearTiemouts();
            ClearBuffers();
        }

        void wclSerialClient_OnError(object Sender, wclSerialError Errors, wclSerialCommunicationState States)
        {
            String Str = "";
            if ((Errors & wclSerialError.erBreak) != 0)
                Str += "erBreak ";
            if ((Errors & wclSerialError.erFrame) != 0)
                Str += "erFrame ";
            if ((Errors & wclSerialError.erOverrun) != 0)
                Str += "erOverrun ";
            if ((Errors & wclSerialError.erRxOver) != 0)
                Str += "erRxOver ";
            if ((Errors & wclSerialError.erRxParity) != 0)
                Str += "erRxParity ";
            lbEvents.Items.Add("Error: " + Str);

            Str = "";
            if ((States & wclSerialCommunicationState.csCtsHold) != 0)
                Str += "csCtsHold ";
            if ((States & wclSerialCommunicationState.csDsrHold) != 0)
                Str += "csDsrHold ";
            if ((States & wclSerialCommunicationState.csRlsdHold) != 0)
                Str += "csRlsdHold ";
            if ((States & wclSerialCommunicationState.csXoffHold) != 0)
                Str += "csXoffHold ";
            if ((States & wclSerialCommunicationState.csXoffSent) != 0)
                Str += "csXoffSent ";
            if ((States & wclSerialCommunicationState.csEof) != 0)
                Str += "csEof ";
            if ((States & wclSerialCommunicationState.csTxim) != 0)
                Str += "csTxim ";
            lbEvents.Items.Add("States: " + Str);
        }

        void wclSerialClient_OnReadError(object Sender, int Error)
        {
            lbEvents.Items.Add("Read error: 0x" + Error.ToString("X8"));
        }

        void wclSerialClient_OnEvents(object Sender, wclSerialEvent Events)
        {
            String Str = "";
            if ((Events & wclSerialEvent.evBreak) != 0)
                Str += "evBreak ";
            if ((Events & wclSerialEvent.evCts) != 0)
                Str += "evCts ";
            if ((Events & wclSerialEvent.evDsr) != 0)
                Str += "evDsr ";
            if ((Events & wclSerialEvent.evRing) != 0)
                Str += "evRing ";
            if ((Events & wclSerialEvent.evRlsd) != 0)
                Str += "evRlsd ";
            if ((Events & wclSerialEvent.evChar) != 0)
                Str += "evChar ";
            lbEvents.Items.Add("Event: " + Str);

            if (Events != 0)
            {
                wclModemStatus Status;
                Int32 Res = wclSerialClient.GetModemStatus(out Status);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    lbEvents.Items.Add("GetModemStatus error: 0x" + Res.ToString("X8"));
                else
                {
                    Str = "";
                    if ((Status & wclModemStatus.msCtsOn) != 0)
                        Str += "msCtsOn ";
                    if ((Status & wclModemStatus.msDsrOn) != 0)
                        Str += "msDsrOn ";
                    if ((Status & wclModemStatus.msRingOn) != 0)
                        Str += "msRingOn ";
                    if ((Status & wclModemStatus.msDsrOn) != 0)
                        Str += "msDsrOn ";
                    if ((Status & wclModemStatus.msRlsdOn) != 0)
                        Str += "msRlsdOn ";
                    if (Str != "")
                        lbEvents.Items.Add("Modem status: " + Str);
                }
            }
        }

        private void fmMain_Closed(object sender, EventArgs e)
        {
            wclSerialClient.Disconnect();
        }

        private void btClear_Click(object sender, RoutedEventArgs e)
        {
            lbEvents.Items.Clear();
        }

        private void ReadConfiguration()
        {
            wclSerialConfig Config;
            Int32 Res = wclSerialClient.GetConfig(out Config);
            if (Res != wclErrors.WCL_E_SUCCESS)
                lbEvents.Items.Add("Read configuration error: 0x" + Res.ToString("X8"));
            else
            {
                edBaudRate.Text = Config.BaudRate.ToString();
                edXonLimit.Text = Config.XonLim.ToString();
                edXoffLimit.Text = Config.XoffLim.ToString();
                edXonChar.Text = Config.XonChar.ToString();
                edXoffChar.Text = Config.XoffChar.ToString();
                edErrorChar.Text = Config.ErrorChar.ToString();
                edEofChar.Text = Config.EofChar.ToString();
                edEventChar.Text = Config.EvtChar.ToString();

                cbParityCheck.IsChecked = Config.ParityCheck;
                cbOutxCtsFlow.IsChecked = Config.OutxCtsFlow;
                cbOutxDsrFlow.IsChecked = Config.OutxDsrFlow;
                cbDsrSensitivity.IsChecked = Config.DsrSensitivity;
                cbTXContinueOnXoff.IsChecked = Config.TXContinueOnXoff;
                cbOutX.IsChecked = Config.OutX;
                cbInX.IsChecked = Config.InX;
                cbErrorCharReplace.IsChecked = Config.ErrorCharReplace;
                cbNullStrip.IsChecked = Config.NullStrip;
                cbAbortOnError.IsChecked = Config.AbortOnError;

                cbRtsControl.SelectedIndex = (Int32)Config.RtsControl;
                cbDtrControl.SelectedIndex = (Int32)Config.DtrControl;
                cbByteSize.SelectedIndex = (Int32)Config.ByteSize - 4;
                cbParity.SelectedIndex = (Int32)Config.Parity;
                cbStopBits.SelectedIndex = (Int32)Config.StopBits;
            }
        }

        private void ReadTiemouts()
        {
            wclSerialTimeouts Times;
            Int32 Res = wclSerialClient.GetTimeouts(out Times);
            if (Res != wclErrors.WCL_E_SUCCESS)
                lbEvents.Items.Add("Get timeouts error: 0x" + Res.ToString("X8"));
            else
            {
                edReadInterval.Text = Times.ReadInterval.ToString();
                edReadMultiplier.Text = Times.ReadMultiplier.ToString();
                edReadConstant.Text = Times.ReadConstant.ToString();
                edWriteMultiplier.Text = Times.WriteMultiplier.ToString();
                edWriteConstant.Text = Times.WriteConstant.ToString();
            }
        }

        private void btEnum_Click(object sender, RoutedEventArgs e)
        {
            EnumComPorts();
        }

        private void EnumComPorts()
        {
            cbPorts.Items.Clear();

            wclSerialDevice[] Ports;
            Int32 Res = wclSerialMonitor.EnumDevices(out Ports);
            if (Res != wclErrors.WCL_E_SUCCESS)
                lbEvents.Items.Add("Error enumerating COM ports: 0x" + Res.ToString("X8"));
            else
            {
                if (Ports != null)
                    for (int i = 0; i < Ports.Length; i++)
                        cbPorts.Items.Add(Ports[i].DeviceName);

                if (cbPorts.Items.Count > 0)
                    cbPorts.SelectedIndex = 0;
                else
                    cbPorts.SelectedIndex = -1;
            }
        }

        private void btConnect_Click(object sender, RoutedEventArgs e)
        {
            if (cbPorts.SelectedIndex == -1)
                MessageBox.Show("Select COM port");
            else
            {
                wclSerialClient.DeviceName = (String)cbPorts.Items[cbPorts.SelectedIndex];

                Int32 Res = wclSerialClient.Connect();
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Error: 0x" + Res.ToString("X8"));
            }
        }

        private void btDisconnect_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = wclSerialClient.Disconnect();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
        }

        private void ClearConfig()
        {
            edBaudRate.Text = "";
            edXoffLimit.Text = "";
            edXonLimit.Text = "";
            edXonChar.Text = "";
            edXoffChar.Text = "";
            edErrorChar.Text = "";
            edEofChar.Text = "";
            edEventChar.Text = "";

            cbParityCheck.IsChecked = false;
            cbOutxCtsFlow.IsChecked = false;
            cbOutxDsrFlow.IsChecked = false;
            cbDsrSensitivity.IsChecked = false;
            cbTXContinueOnXoff.IsChecked = false;
            cbOutX.IsChecked = false;
            cbInX.IsChecked = false;
            cbErrorCharReplace.IsChecked = false;
            cbNullStrip.IsChecked = false;
            cbAbortOnError.IsChecked = false;

            cbRtsControl.SelectedIndex = -1;
            cbDtrControl.SelectedIndex = -1;
            cbByteSize.SelectedIndex = -1;
            cbParity.SelectedIndex = -1;
            cbStopBits.SelectedIndex = -1;
        }

        private void btSetConfig_Click(object sender, RoutedEventArgs e)
        {
            wclSerialConfig Config = new wclSerialConfig();

            Config.BaudRate = Convert.ToUInt32(edBaudRate.Text);
            Config.XonLim = Convert.ToUInt16(edXonLimit.Text);
            Config.XoffLim = Convert.ToUInt16(edXoffLimit.Text);
            Config.XonChar = Convert.ToByte(edXonChar.Text);
            Config.XoffChar = Convert.ToByte(edXoffChar.Text);
            Config.ErrorChar = Convert.ToByte(edErrorChar.Text);
            Config.EofChar = Convert.ToByte(edEofChar.Text);
            Config.EvtChar = Convert.ToByte(edEventChar.Text);

            Config.ParityCheck = cbParityCheck.IsChecked == true;
            Config.OutxCtsFlow = cbOutxCtsFlow.IsChecked == true;
            Config.OutxDsrFlow = cbOutxDsrFlow.IsChecked == true;
            Config.DsrSensitivity = cbDsrSensitivity.IsChecked == true;
            Config.TXContinueOnXoff = cbTXContinueOnXoff.IsChecked == true;
            Config.OutX = cbOutX.IsChecked == true;
            Config.InX = cbInX.IsChecked == true;
            Config.ErrorCharReplace = cbErrorCharReplace.IsChecked == true;
            Config.NullStrip = cbNullStrip.IsChecked == true;
            Config.AbortOnError = cbAbortOnError.IsChecked == true;

            Config.RtsControl = (wclSerialRtsControl)cbRtsControl.SelectedIndex;
            Config.DtrControl = (wclSerialDtrControl)cbDtrControl.SelectedIndex;
            Config.ByteSize = (Byte)(cbByteSize.SelectedIndex + 4);
            Config.Parity = (wclSerialParity)cbParity.SelectedIndex;
            Config.StopBits = (wclSerialStopBits)cbStopBits.SelectedIndex;

            Int32 Res = wclSerialClient.SetConfig(Config);
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
        }

        private void btHetconfig_Click(object sender, RoutedEventArgs e)
        {
            ReadConfiguration();
        }

        private void ClearBuffers()
        {
            edReadBufferSize.Text = "";
            edWriteBufferSize.Text = "";
        }

        private void ClearTiemouts()
        {
            edReadInterval.Text = "";
            edReadMultiplier.Text = "";
            edReadConstant.Text = "";
            edWriteMultiplier.Text = "";
            edWriteConstant.Text = "";
        }

        private void ReadBuffers()
        {
            UInt32 Size;
            Int32 Res = wclSerialClient.GetReadBufferSize(out Size);
            if (Res != wclErrors.WCL_E_SUCCESS)
                lbEvents.Items.Add("Get read buffer size error: 0x" + Res.ToString("X8"));
            else
                edReadBufferSize.Text = Size.ToString();

            Res = wclSerialClient.GetWriteBufferSize(out Size);
            if (Res != wclErrors.WCL_E_SUCCESS)
                lbEvents.Items.Add("Get write buffer size error: 0x" + Res.ToString("X8"));
            else
                edWriteBufferSize.Text = Size.ToString();
        }

        private void btGetBuffer_Click(object sender, RoutedEventArgs e)
        {
            ReadBuffers();
        }

        private void btSetBuffers_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = wclSerialClient.SetReadBufferSize(Convert.ToUInt32(edReadBufferSize.Text));
            if (Res != wclErrors.WCL_E_SUCCESS)
                lbEvents.Items.Add("Set read buffer size error: 0x" + Res.ToString("X8"));

            Res = wclSerialClient.SetWriteBufferSize(Convert.ToUInt32(edWriteBufferSize.Text));
            if (Res != wclErrors.WCL_E_SUCCESS)
                lbEvents.Items.Add("Set write buffer size error: 0x" + Res.ToString("X8"));
        }

        private void btGetTimeouts_Click(object sender, RoutedEventArgs e)
        {
            ReadTiemouts();
        }

        private void btSetTimeouts_Click(object sender, RoutedEventArgs e)
        {
            wclSerialTimeouts Times = new wclSerialTimeouts();
            Times.ReadInterval = Convert.ToUInt32(edReadInterval.Text);
            Times.ReadMultiplier = Convert.ToUInt32(edReadMultiplier.Text);
            Times.ReadConstant = Convert.ToUInt32(edReadConstant.Text);
            Times.WriteMultiplier = Convert.ToUInt32(edWriteMultiplier.Text);
            Times.WriteConstant = Convert.ToUInt32(edWriteConstant.Text);
            Int32 Res = wclSerialClient.SetTimeouts(Times);
            if (Res != wclErrors.WCL_E_SUCCESS)
                lbEvents.Items.Add("Set timeouts error: 0x" + Res.ToString("X8"));
        }

        private void btClearCommBreak_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = wclSerialClient.ClearCommBreak();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
        }

        private void btSetCommBreak_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = wclSerialClient.SetCommBreak();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
        }

        private void btFlushBuffers_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = wclSerialClient.FlushBuffers();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));  
        }

        private void btExecFunc_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = wclSerialClient.EscapeCommFunction((wclSerialEscapeFunction)cbFunction.SelectedIndex);
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
        }

        private void btPurge_Click(object sender, RoutedEventArgs e)
        {
            wclSerialPurgeFlag Flags = 0;
            if (cbpurgeRxAbort.IsChecked == true)
                Flags |= wclSerialPurgeFlag.purgeRxAbort;
            if (cbpurgeRxClear.IsChecked == true)
                Flags |= wclSerialPurgeFlag.purgeRxClear;
            if (cbpurgeTxAbort.IsChecked == true)
                Flags |= wclSerialPurgeFlag.purgeTxAbort;
            if (cbpurgeTxClear.IsChecked == true)
                Flags |= wclSerialPurgeFlag.purgeTxClear;

            Int32 Res = wclSerialClient.PurgeComm(Flags);
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
        }

        private void btSebd_Click(object sender, RoutedEventArgs e)
        {
            String s = edText.Text;
            switch (cbLineFeed.SelectedIndex)
            {
                case 1:
                    s += '\r';
                    break;
                case 2:
                    s += '\n';
                    break;
                case 3:
                    s += "\r\n";
                    break;
            }
            Byte[] Ansi = Encoding.ASCII.GetBytes(s);
            UInt32 Sent = 0;
            Int32 Res = wclSerialClient.Write(Ansi, out Sent);
            lbEvents.Items.Add("Sent: " + Sent.ToString() + " bytes from " + Ansi.Length.ToString());
            if (Res != wclErrors.WCL_E_SUCCESS)
                lbEvents.Items.Add("Write error: 0x" + Res.ToString("X8"));
        }

        private void btTransmit_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = wclSerialClient.TransmitCommChar(Convert.ToByte(edCharCode.Text));
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Error: 0x" + Res.ToString("X8"));
        }

        private void btSetWriteTimeout_Click(object sender, RoutedEventArgs e)
        {
            wclSerialClient.WriteTimeout = Convert.ToUInt32(edWriteTimeout.Text);
        }
    }
}
