﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using wclSerial;
using wclCommon;

namespace SerialMonitor
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private wclSerialMonitor wclSerialMonitor;
        private wclPowerEventsMonitor FPowerMonitor;

        internal class DeviceItem
        {
            public String FriendlyName { get; set; }
            public String IsModem { get; set; }
            public String DeviceName { get; set; }
        };

        public MainWindow()
        {
            InitializeComponent();
        }

        private void btClear_Click(object sender, RoutedEventArgs e)
        {
            lbEvents.Items.Clear();
        }

        private void fmMain_Loaded(object sender, RoutedEventArgs e)
        {
            wclSerialMonitor = new wclSerialMonitor();
            wclSerialMonitor.OnInserted += new wclSerialDeviceEvent(wclSerialMonitor_OnInserted);
            wclSerialMonitor.OnRemoved += new wclSerialDeviceEvent(wclSerialMonitor_OnRemoved);

            FPowerMonitor = new wclPowerEventsMonitor();
            FPowerMonitor.OnPowerStateChanged += new wclPowerStateChangedEvent(FPowerMonitor_OnPowerStateChanged);
            FPowerMonitor.Open();
        }

        private Boolean Running = false;

        void FPowerMonitor_OnPowerStateChanged(object Sender, wclPowerState State)
        {
            switch (State)
            {
                case wclPowerState.psResumeAutomatic:
                    if (Running)
                    {
                        Running = false;
                        wclSerialMonitor.Start();
                    }
                    lbEvents.Items.Add("Power changed: psResumeAutomatic");
                    break;

                case wclPowerState.psResume:
                    lbEvents.Items.Add("Power changed: psResume");
                    break;

                case wclPowerState.psSuspend:
                    if (wclSerialMonitor.Monitoring)
                    {
                        Running = true;
                        wclSerialMonitor.Stop();
                    }
                    lbEvents.Items.Add("Power changed: psSuspend");
                    break;

                case wclPowerState.psUnknown:
                    lbEvents.Items.Add("Power changed: psUnknown");
                    break;
            }
        }

        void wclSerialMonitor_OnInserted(object Sender, wclSerialDevice Device)
        {
            lbEvents.Items.Add("Device inserted:");
            lbEvents.Items.Add("Friendly name: " + Device.FriendlyName);
            lbEvents.Items.Add("  Modem: " + Device.IsModem.ToString());
            lbEvents.Items.Add("  DeviceName: " + Device.DeviceName);
        }

        void wclSerialMonitor_OnRemoved(object Sender, wclSerialDevice Device)
        {
            lbEvents.Items.Add("Device removed:");
            lbEvents.Items.Add("Friendly name: " + Device.FriendlyName);
            lbEvents.Items.Add("  Modem: " + Device.IsModem.ToString());
            lbEvents.Items.Add("  DeviceName: " + Device.DeviceName);
        }

        private void fmMain_Closed(object sender, EventArgs e)
        {
            wclSerialMonitor.Stop();
            FPowerMonitor.Close();
        }

        private void btEnum_Click(object sender, RoutedEventArgs e)
        {
            lvDevices.Items.Clear();

            wclSerialDevice[] Devices;
            Int32 Res = wclSerialMonitor.EnumDevices(out Devices);
            lbEvents.Items.Add("Enumerate serial devices: 0x" + Res.ToString("X8"));

            if (Res == wclErrors.WCL_E_SUCCESS)
            {
                if (Devices != null)
                {
                    for (int i = 0; i < Devices.Length; i++)
                    {
                        lvDevices.Items.Add(new DeviceItem {
                            FriendlyName = Devices[i].FriendlyName,
                            IsModem = Devices[i].IsModem.ToString(),
                            DeviceName = Devices[i].DeviceName
                        });
                    }
                }
            }
        }

        private void btStart_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = wclSerialMonitor.Start();
            lbEvents.Items.Add("Monitoring started: 0x" + Res.ToString("X8"));
        }

        private void btStop_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = wclSerialMonitor.Stop();
            lbEvents.Items.Add("Monitoring stopped: 0x" + Res.ToString("X8"));
        }
    }
}
