﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using wclCommon;
using wclSerial;

namespace UsbMonitor
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private wclUsbMonitor Monitor;

        internal class DeviceItem
        {
            public String FriendlyName { get; set; }
            public String HardwareId { get; set; }
            public String VendorId { get; set; }
            public String ProductId { get; set; }
            public String Revision { get; set; }
            public String Manufacturer { get; set; }
            public String Instance { get; set; }
            public String ClassGuid { get; set; }
            public String Enabled { get; set; }
        };

        public MainWindow()
        {
            InitializeComponent();
        }

        private void btEnum_Click(object sender, RoutedEventArgs e)
        {
            lvDevices.Items.Clear();

            wclUsbDevice[] Devices;
            Int32 Res = Monitor.EnumDevices(out Devices);
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Unumeration failed: 0x" + Res.ToString("X8"));
            else
            {
                if (Devices != null)
                {
                    foreach (wclUsbDevice Device in Devices)
                    {
                        lvDevices.Items.Add(new DeviceItem {
                            FriendlyName = Device.FriendlyName,
                            HardwareId = Device.HardwareId,
                            VendorId = Device.VendorId.ToString("X4"),
                            ProductId = Device.ProductId.ToString("X4"),
                            Revision = Device.Revision.ToString("X4"),
                            Manufacturer = Device.Manufacturer,
                            Instance = Device.Instance,
                            ClassGuid = Device.ClassGuid.ToString(),
                            Enabled = Device.Enabled.ToString()
                        });
                    }
                }
            }
        }

        private void fmMain_Loaded(object sender, RoutedEventArgs e)
        {
            Monitor = new wclUsbMonitor();
            Monitor.OnInserted += new wclUsbDeviceEvent(Monitor_OnInserted);
            Monitor.OnRemoved += new wclUsbDeviceEvent(Monitor_OnRemoved);
        }

        void Monitor_OnInserted(object Sender, wclUsbDevice Device)
        {
            lbEvents.Items.Add("Device inserted: " + Device.Instance);
        }

        void Monitor_OnRemoved(object Sender, wclUsbDevice Device)
        {
            lbEvents.Items.Add("Device removed: " + Device.Instance);
        }

        private void fmMain_Closed(object sender, EventArgs e)
        {
            Monitor.Stop();
        }

        private void btClear_Click(object sender, RoutedEventArgs e)
        {
            lbEvents.Items.Clear();
        }

        private void btStart_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = Monitor.Start();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Start monitoring failed: 0x" + Res.ToString("X8"));
            else
                lbEvents.Items.Add("Monitoring started");
        }

        private void btStop_Click(object sender, RoutedEventArgs e)
        {
            Int32 Res = Monitor.Stop();
            if (Res != wclErrors.WCL_E_SUCCESS)
                MessageBox.Show("Stop monitoring failed: 0x" + Res.ToString("X8"));
            else
                lbEvents.Items.Add("Monitoring stopped");
        }

        private void btEnable_Click(object sender, RoutedEventArgs e)
        {
            if (lvDevices.SelectedItems.Count == 0)
                MessageBox.Show("Select device");
            else
            {
                DeviceItem Item = (DeviceItem)lvDevices.SelectedItems[0];
                Int32 Res = Monitor.Enable(Item.Instance);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Failed to enable device: 0x" + Res.ToString("X8"));
            }
        }

        private void btDisable_Click(object sender, RoutedEventArgs e)
        {
            if (lvDevices.SelectedItems.Count == 0)
                MessageBox.Show("Select device");
            else
            {
                DeviceItem Item = (DeviceItem)lvDevices.SelectedItems[0];
                Int32 Res = Monitor.Disable(Item.Instance);
                if (Res != wclErrors.WCL_E_SUCCESS)
                    MessageBox.Show("Failed to disable device: 0x" + Res.ToString("X8"));
            }
        }
    }
}
